use std::collections::HashMap;
use std::fs;
use std::io::{self, Read};
use regex::Regex;
use pyo3::Python;
use pyo3::types::PyModule;
use pyo3::PyResult;
use pyo3::pymodule;
use std::fs::File;
use std::io::Write;
use std::path::Path;


#[pymodule]
fn text_analysis(_py: Python, m: &PyModule) -> PyResult<()> {

    #[pyfn(m)]
    fn calc_statistics<'py>(_py: Python<'py>, file_path: &str) -> PyResult<&'static str> {
        match analyze_file(file_path) {
            Ok((sentence_count, word_count, char_count, frequent_words, words_array)) => {
                let _ = write_statistics(file_path, sentence_count, word_count, char_count, frequent_words, words_array);
                Ok("Success")
            },
            Err(e) => {
                Err(pyo3::exceptions::PyRuntimeError::new_err(format!("Error reading file {}: {}", file_path, e)))
            }
        }
    }

    Ok(())
}

fn find_repeated_phrases(words: &Vec<String>, n: usize) -> HashMap<String, usize> {
    let mut phrases: HashMap<String, usize> = HashMap::new();
    for i in 0..(words.len() - n + 1) {
        let phrase = words[i..i + n].join(" ");
        *phrases.entry(phrase).or_insert(0) += 1;
    }
    phrases.into_iter().filter(|&(_, count)| count > 1).collect()
}

fn top_n_phrases(phrases: HashMap<String, usize>, n: usize) -> Vec<(String, usize)> {
    let mut items: Vec<(String, usize)> = phrases.into_iter().collect();
    items.sort_by(|a, b| b.1.cmp(&a.1));
    items.into_iter().take(n).collect()
}

fn analyze_file(file_path: &str) -> io::Result<(usize, usize, usize, HashMap<String, usize>, Vec<String>)> {
    let mut text = String::new();
    fs::File::open(file_path)?.read_to_string(&mut text)?;

    let char_count = text.chars().count();
    let re = Regex::new(r"[^\w\s']").unwrap();
    let clean_text = re.replace_all(&text, "").to_lowercase();
    let words_array: Vec<String> = clean_text.split_whitespace().map(|s| s.to_string()).collect();
    let word_count = words_array.len();

    let mut frequent_words = HashMap::new();
    for word in &words_array {
        *frequent_words.entry(word.clone()).or_insert(0) += 1;
    }

    let sentence_array: Vec<&str> = text.split('.').filter(|s| !s.is_empty()).collect();
    let sentence_count = sentence_array.len();

    Ok((sentence_count, word_count, char_count, frequent_words, words_array))
}

fn write_statistics(file_path: &str, sentence_count: usize, word_count: usize, char_count: usize, frequent_words: HashMap<String, usize>, words_array: Vec<String>) -> std::io::Result<()> {
    let file_name = Path::new(file_path).file_name().unwrap().to_str().unwrap();
    let output_file_name = format!("{}_statistics.txt", file_name);
    let mut file = File::create(output_file_name)?;
    
    writeln!(file, "File {} statistics:\n", file_path)?;
    writeln!(file, "Number of sentences: {}", sentence_count)?;
    writeln!(file, "Number of words: {}", word_count)?;
    writeln!(file, "Number of characters: {}", char_count)?;
    writeln!(file, "Average sentence length in words: {:.2}", word_count as f64 / sentence_count as f64)?;
    writeln!(file, "Average sentence length in characters: {:.2}", char_count as f64 / sentence_count as f64)?;

    if let Some(most_frequent_word) = frequent_words.iter().max_by_key(|entry| entry.1) {
        writeln!(file, "Most frequent word: {:?}", most_frequent_word)?;
    }

    if let Some(most_frequent_long_word) = frequent_words.iter().filter(|(word, _)| word.len() > 4).max_by_key(|entry| entry.1) {
        writeln!(file, "Most frequent word longer than 3 characters: {:?}", most_frequent_long_word)?;
    }

    let most_frequent_3_phrases = find_repeated_phrases(&words_array, 3);
    writeln!(file, "Most frequent 3-words phrases: {:?}", top_n_phrases(most_frequent_3_phrases, 5))?;

    let most_frequent_4_phrases = find_repeated_phrases(&words_array, 4);
    writeln!(file, "Most frequent 4-words phrases: {:?}", top_n_phrases(most_frequent_4_phrases, 5))?;

    let most_frequent_5_phrases = find_repeated_phrases(&words_array, 5);
    writeln!(file, "Most frequent 5-words phrases: {:?}", top_n_phrases(most_frequent_5_phrases, 5))?;

    let most_frequent_6_phrases = find_repeated_phrases(&words_array, 6);
    writeln!(file, "Most frequent 6-words phrases: {:?}", top_n_phrases(most_frequent_6_phrases, 5))?;

    Ok(())
}